/*
 * pftp -- sends files from host to host through free choosable ports
 *
 * Copyright (C) 1996-1999 Ben Schluricke
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the emplied warranty of MERCHANT-
 * ABILITY OF FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *    Written by Ben Schluricke
 *    E-Mail:    bhor0533@lehr.chem.TU-Berlin.DE
 *
 * This program is dedicated to my girl-friend, Heather O'Rourke.
 *
 *
 */
#ifdef USE_POSIX_THREAD
#define _REENTRANT
#include <pthread.h>
#endif
#ifdef FreeBSD
#include <sys/errno.h>
#endif
#ifdef unicos
#include <sys/types.h>
#endif
#include "main.h"

extern void check_bandwidth(size_t);
extern void timing(int, double *, char *);

void receive_udps(short strnum, double *ci)
{
#ifdef HAVE_INET6
   struct sockaddr_in6 *sin = &(*statstr)->sin;
   int ret=0;
#else
   struct sockaddr_in *sin = &(*statstr)->sin;
   struct hostent *hp;
#endif
   char reset_timing_ = 1;
   register int num=0, check=0;
   int sinlen = (*statstr)->sinlen;
   int strlength = (*statstr)->bsize;
#ifdef Linux
   unsigned char str[strlength];
#else
   unsigned char str[BIG_BUFSIZE];
#endif
   unsigned char *strptr=NULL;
   int length=0;
   
   /*
    * Receive data UDP.
    */
   for (num=1, *ci=0.0; num; *ci+=(double)num) {
      if ((num = recvfrom(s, (void *)str, strlength, 0, (struct sockaddr *)sin, &sinlen)) < 0) {
         if (slfp) fprintf(slfp, "** recvfrom: %s\n", _PFTP_ERROR_ARRAY_);
         exit(1);
      }
      if (reset_timing_) {
         if (slfp && !(*(statstr+strnum))->first) {
#ifdef HAVE_INET6
            if (!(*statstr)->multicast) {
               if ((ret = getnameinfo((struct sockaddr *)sin, sinlen, \
                  (*(statstr+strnum))->REMOTEHOSTNAME, SONAME*sizeof(char), NULL, 0, 0))) {
                  if (!inet_ntop(sin->sin6_family, sin->sin6_addr.s6_addr, (*(statstr+strnum))->REMOTEHOSTNAME, INET6_ADDRSTRLEN)) {
                     fprintf(slfp, "Cannot convert binary address string.\n");
                     if (shutdown(s, 2) < 0) {
                        if (slfp) fprintf(slfp, "** shutdown: %s\n", _PFTP_ERROR_ARRAY_);
                        exit(1);
                     }
                  }
               }
            }
            else strcpy((*(statstr+strnum))->REMOTEHOSTNAME, (*statstr)->_MULTICASTGROUP_);
#else
#if defined IPV6_ADD_MEMBERSHIP || (defined IP_ADD_MEMBERSHIP && !defined HAVE_INET6)
            if (!(*statstr)->multicast) {
#endif
               if ((hp = gethostbyaddr((char *)&sin->sin_addr, sizeof(sin->sin_addr), AF_INET)) == 0) {
                  strcpy((*(statstr+strnum))->REMOTEHOSTNAME, inet_ntoa(sin->sin_addr));
               }
               else {
                  strcpy((*(statstr+strnum))->REMOTEHOSTNAME, hp->h_name);
               }
#if defined IPV6_ADD_MEMBERSHIP || (defined IP_ADD_MEMBERSHIP && !defined HAVE_INET6)
            }
            else {
               strcpy((*(statstr+strnum))->REMOTEHOSTNAME, (*statstr)->_MULTICASTGROUP_);
            }
#endif
#endif
            fprintf(slfp, "* Receiving datagrams from %s.\n\r", (*(statstr+strnum))->REMOTEHOSTNAME);
            timing(strnum, ci, NULL);
            (*(statstr+strnum))->first = BIT_ONE;
         }
         else if (slfp && ((*(statstr+strnum))->first & BIT_ONE)) {
            timing(strnum, ci, NULL);
            reset_timing_ = 0;
#ifdef USE_POSIX_THREAD
            CHILDNUM++;
#endif
         }
      }
#ifdef Linux
      if ((*statstr)->show_bytes) fprintf(slfp, "\r* Bytes: %.0f.", *ci);
#endif
      if (!(*statstr)->testbsize) {
         for (check=0, strptr=str, length=num; length; length-=check, strptr += check) {
            if ((check = write(1, strptr, length)) < 0) {
               if (slfp) fprintf(slfp, "** write: %s\n", _PFTP_ERROR_ARRAY_);
               exit(1);
            }
         }
      }
   }
   (*(statstr+strnum))->first = BIT_TWO; 
}

void send_udps(int ft, double *ci)
{
#ifdef HAVE_INET6
   struct sockaddr_in6 *sin = (struct sockaddr_in6 *)(*statstr)->adin->ai_addr;
#else
   struct sockaddr_in *sin = &(*statstr)->sin;
#endif
   int num=0, nn=0;
   unsigned long i=0;
   double count=0, max=(*statstr)->testbsize;
   int sinlen = (*statstr)->sinlen;
   int strlength = (*statstr)->bsize;
   char *tbuffer=NULL;
#ifdef Linux
   unsigned char str[strlength];
#else
   unsigned char str[BUFSIZE];
#endif
   if (max) {
      tbuffer = malloc(strlength*sizeof(char));
      for (count=0.0; count < max; count+=(double)nn, check_bandwidth(nn)) {
         if (max - count < (double)strlength) strlength = (int)(max - count);
         if ((nn = sendto(s, tbuffer, strlength, 0, (struct sockaddr *)sin, sinlen)) < 0) {
            if (slfp) fprintf(slfp, "\n\r** sendto: %s\n", _PFTP_ERROR_ARRAY_);
            exit(1);
         }
      }
      sendto(s, tbuffer, 0, 0, (struct sockaddr *)sin, sinlen);
   }
   else {
      /*
       * Send data via UDP.
       */
#if defined Linux
      (*statstr)->new_line = 1;
#endif
      for (num=1; num; count+=(double)nn, check_bandwidth(nn), i++) {
         if ((num = read(ft, str, strlength)) < 0) {
            if (slfp) fprintf(slfp, "\n\r** read: %s\n", _PFTP_ERROR_ARRAY_);
            exit(1);
         }
         if ((*statstr)->throw_away && !(i % ((*statstr)->throw_away))) {
            i = 0; continue;
         }
         if ((nn = sendto(s, (void *)str, num, 0, (struct sockaddr *)sin, sinlen)) < 0) {
            if (slfp) fprintf(slfp, "\n\r** sendto: %s\n", _PFTP_ERROR_ARRAY_);
            exit(1);
         }
      }
   }
   *ci = count;
}
